/* XPM */
static const char *line1[] =
{
/* width height ncolors chars_per_pixel */
"12 12 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"ABBBBBBBBBBA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
};

/* XPM */
static const char *line2[] =
{
/* width height ncolors chars_per_pixel */
"12 12 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
};

/* XPM */
static const char *line3[] =
{
/* width height ncolors chars_per_pixel */
"12 12 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
};

/* XPM */
static const char *line4[] =
{
/* width height ncolors chars_per_pixel */
"12 12 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
};

/* XPM */
static const char *line5[] =
{
/* width height ncolors chars_per_pixel */
"12 12 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"ABBBBBBBBBBA",
"AAAAAAAAAAAA",
"AAAAAAAAAAAA",
};
