#include "chemdata.h"
#include "dpoint.h"
#include "text.h"
#include "defs.h"

bool ChemData::load_mdl( QString fn )
{
    return false;
}

bool ChemData::ProcessMDL( QString wf )
{
    return false;
}

bool ChemData::save_mdl( QString fn )
{
    return false;
}

// kate: tab-width 4; indent-width 4; space-indent on; replace-trailing-space-save on;
