/* XPM */
static const char *n3ring[] =
{
/* width height ncolors chars_per_pixel */
"24 24 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAABAAAAAAAAAAAAAAAAA",
"AAAAAABBBAAAAAAAAAAAAAAA",
"AAAAAABAABBAAAAAAAAAAAAA",
"AAAAAABAAAABAAAAAAAAAAAA",
"AAAAAABAAAAABBAAAAAAAAAA",
"AAAAAABAAAAAAABBAAAAAAAA",
"AAAAAABAAAAAAAAABBAAAAAA",
"AAAAAABAAAAAAAAAAABBAAAA",
"AAAAAABAAAAAAAAAAABAAAAA",
"AAAAAABAAAAAAAAABBAAAAAA",
"AAAAAABAAAAAAABBAAAAAAAA",
"AAAAAABAAAAAABAAAAAAAAAA",
"AAAAAABAAAABBAAAAAAAAAAA",
"AAAAAABAABBAAAAAAAAAAAAA",
"AAAAAABABAAAAAAAAAAAAAAA",
"AAAAAABBAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
};
/* XPM */
static const char *n4ring[] =
{
/* width height ncolors chars_per_pixel */
"24 24 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAABBBBBBBBBBBBBBBBAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABAAAAAAAAAAAAAABAAAA",
"AAAABBBBBBBBBBBBBBBBAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
};
/* XPM */
static const char *n5ring[] =
{
/* width height ncolors chars_per_pixel */
"24 24 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAABAAAAAAAAA",
"AAAAAAAAAAABBBABAAAAAAAA",
"AAAAAAAABBBAAAAABAAAAAAA",
"AAAAABBBAAAAAAAABAAAAAAA",
"AAABBAAAAAAAAAAAABAAAAAA",
"AAABAAAAAAAAAAAAAABAAAAA",
"AAABAAAAAAAAAAAAAAABAAAA",
"AAABAAAAAAAAAAAAAAABAAAA",
"AAABAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAABAA",
"AAABAAAAAAAAAAAAAAAAABAA",
"AAABAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAABAAAA",
"AAABAAAAAAAAAAAAAAABAAAA",
"AAABAAAAAAAAAAAAAABAAAAA",
"AAABAAAAAAAAAAAAABAAAAAA",
"AAABAAAAAAAAAAAABAAAAAAA",
"AAAABBBAAAAAAAAABAAAAAAA",
"AAAAAAABBBAAAAABAAAAAAAA",
"AAAAAAAAAABBBABAAAAAAAAA",
"AAAAAAAAAAAAABBAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
};
/* XPM */
static const char *n5ring_diene[] =
{
/* width height ncolors chars_per_pixel */
"24 24 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAABAAAAAAAAA",
"AAAAAAAAAAABBBABAAAAAAAA",
"AAAAAAAABBBAAAAABAAAAAAA",
"AAAAABBBAAAAAAAABAAAAAAA",
"AAABBAAAAAAAAAAAABAAAAAA",
"AAABAAAAAAAAAAAAAABAAAAA",
"AAABABAAAAAAAAAAAAABAAAA",
"AAABABAAAAAAAAAAAAABAAAA",
"AAABABAAAAAAAAAAAAAABAAA",
"AAABABAAAAAAAAAAAAAAABAA",
"AAABABAAAAAAAAAAAAABABAA",
"AAABABAAAAAAAAAAAABABAAA",
"AAABABAAAAAAAAAAABABAAAA",
"AAABABAAAAAAAAAABAABAAAA",
"AAABABAAAAAAAAABAABAAAAA",
"AAABAAAAAAAAAABAABAAAAAA",
"AAABAAAAAAAAABAABAAAAAAA",
"AAAABBBAAAAAABAABAAAAAAA",
"AAAAAAABBBAAAAABAAAAAAAA",
"AAAAAAAAAABBBABAAAAAAAAA",
"AAAAAAAAAAAAABBAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
};
/* XPM */
static const char *n6ring[] =
{
/* width height ncolors chars_per_pixel */
"24 24 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAABAAAAAAAAAAAA",
"AAAAAAAAABBABBAAAAAAAAAA",
"AAAAAAAABAAAAABAAAAAAAAA",
"AAAAAABBAAAAAAABBAAAAAAA",
"AAAABBAAAAAAAAAAABBAAAAA",
"AABBAAAAAAAAAAAAAAABBAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAABAAA",
"AAABBAAAAAAAAAAAAABBAAAA",
"AAAAABAAAAAAAAAAABAAAAAA",
"AAAAAABBAAAAAAABBAAAAAAA",
"AAAAAAAABBAAABBAAAAAAAAA",
"AAAAAAAAAABBBAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
};
/* XPM */
static const char *n6ring_benzene[] =
{
/* width height ncolors chars_per_pixel */
"24 24 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAABAAAAAAAAAAAA",
"AAAAAAAAABBABBAAAAAAAAAA",
"AAAAAAAABAABAABAAAAAAAAA",
"AAAAAABBAAAABBABBAAAAAAA",
"AAAABBAAAAAAAABAABBAAAAA",
"AABBAAAAAAAAAAABBAABBAAA",
"AABABAAAAAAAAAAAABBABAAA",
"AABABAAAAAAAAAAAAAAABAAA",
"AABABAAAAAAAAAAAAAAABAAA",
"AABABAAAAAAAAAAAAAAABAAA",
"AABABAAAAAAAAAAAAAAABAAA",
"AABABAAAAAAAAAAAAAAABAAA",
"AABABAAAAAAAAAAAAAAABAAA",
"AABABAAAAAAAAAAAAAAABAAA",
"AABABAAAAAAAAAAAAABABAAA",
"AABAAAAAAAAAAAAABBAABAAA",
"AAABBAAAAAAAAAABAABBAAAA",
"AAAAABAAAAAAABBAABAAAAAA",
"AAAAAABBAAABBAABBAAAAAAA",
"AAAAAAAABBAAABBAAAAAAAAA",
"AAAAAAAAAABBBAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
};
/* XPM */
static const char *n6ring_chair[] =
{
/* width height ncolors chars_per_pixel */
"24 24 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAABBAAAAAAAAAAAAABBAA",
"AAAABAABBBBAAAAABBBBBAAA",
"AAAABAAAAAABBBBBAAAABAAA",
"AAABAAAAAAAAAAAAAAABAAAA",
"AABAAAAAABAAAAAAAABAAAAA",
"AABAAABBBABBBAAAAABAAAAA",
"ABABBBAAAAAAABBBABAAAAAA",
"ABBAAAAAAAAAAAAABBAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAA",
};
/* XPM */
static const char *n6ring_boat[] = {
/* width height num_colors chars_per_pixel */
"    24    24        2            1",
/* colors */
". c #000000",
"# c #ffffff",
/* pixels */
"########################",
"########################",
"########################",
"########################",
"########################",
"########################",
"########################",
"###.##############.#####",
"###...###########..#####",
"###.#..##########..#####",
"####.##..#######.#.#####",
"####.###..######.#.#####",
"####.#####........######",
"####.##########.########",
"#####.########.#########",
"#####.#######..#########",
"#####.........##########",
"########################",
"########################",
"########################",
"########################",
"########################",
"########################",
"########################"
};
