static const char *boxbracket_xpm[] =
{
/* width height ncolors chars_per_pixel */
"30 30 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAABBBBBBBBBBBBBBBBBBBBBBBBAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABBBBBBBBBBBBBBBBBBBBBBBBAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
};

static const char *closedsquarebracket_xpm[] =
{
/* width height ncolors chars_per_pixel */
"30 30 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAABBBBBBBBBBBBBBBBBBBBBBBBAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAABAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAABAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAABAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAABAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAABAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAABABABABABABABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABBBBBBBBBBBBBBBBBBBBBBBBAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
};

/* XPM */
static const char *squarebracket_xpm[] =
{
/* width height ncolors chars_per_pixel */
"30 30 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAABBBBBBAAAAAAAAAAAABBBBBBAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABBBBBBAAAAAAAAAAAABBBBBBAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
};

static const char *ellipsebracket_xpm[] =
{
/* width height ncolors chars_per_pixel */
"30 30 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAABBBBBBBBBBBBBBBBAAAAAAA",
"AAAAAABAAAAAAAAAAAAAAAABAAAAAA",
"AAAAABAAAAAAAAAAAAAAAAAABAAAAA",
"AAAABAAAAAAAAAAAAAAAAAAAABAAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAAABAAAAAAAAAAAAAAAAAAAABAAAA",
"AAAAABAAAAAAAAAAAAAAAAAABAAAAA",
"AAAAAABAAAAAAAAAAAAAAAABAAAAAA",
"AAAAAAABBBBBBBBBBBBBBBBAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
};

static const char *circlebracket_xpm[] =
{
/* width height ncolors chars_per_pixel */
"30 30 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAABBBBBBBBBBBBBBBBAAAAAAA",
"AAAAAABAAAAAAAAAAAAAAAABAAAAAA",
"AAAAABAAAAAAAAABAAAAAAAABAAAAA",
"AAAABAAAAAAAAAAAAAAAAAAAABAAAA",
"AAABAAAAAAAAAAABAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAABAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAABAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAABAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAABABABABABABABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAAABAAAAAAAAAAAAAAAAAAAABAAAA",
"AAAAABAAAAAAAAAAAAAAAAAABAAAAA",
"AAAAAABAAAAAAAAAAAAAAAABAAAAAA",
"AAAAAAABBBBBBBBBBBBBBBBAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
};

static const char *curvebracket_xpm[] =
{
/* width height ncolors chars_per_pixel */
"30 30 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAABAAAAAAAAAAAAAABAAAAAAA",
"AAAAAABAAAAAAAAAAAAAAAABAAAAAA",
"AAAAABAAAAAAAAAAAAAAAAAABAAAAA",
"AAAABAAAAAAAAAAAAAAAAAAAABAAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AABAAAAAAAAAAAAAAAAAAAAAAAABAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAAABAAAAAAAAAAAAAAAAAAAABAAAA",
"AAAAABAAAAAAAAAAAAAAAAAABAAAAA",
"AAAAAABAAAAAAAAAAAAAAAABAAAAAA",
"AAAAAAABAAAAAAAAAAAAAABAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
};

static const char *bracebracket_xpm[] =
{
/* width height ncolors chars_per_pixel */
"30 30 2 1",
/* colors */
"A c #FFFFFFFFFFFF",
"B c #000000000000",
/* pixels */
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAABBBBBBAAAAAAAAAAAABBBBBBAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"BBBBAAAAAAAAAAAAAAAAAAAAAABBBB",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABAAAAAAAAAAAAAAAAAAAAAABAAA",
"AAABBBBBBAAAAAAAAAAAABBBBBBAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
};
